using GWanalysis
using DataFrames
using GLMakie
using CairoMakie
using Printf
GLMakie.activate!()


include("../rootdirs.jl")
include("../theme.jl")


#######################################################################
#                         Plotting functions                          #
#######################################################################


function plot_output_0d!(fig, df, var, ylabel, M;
        use_log=false, use_diff=false, use_norm_t0=false)

    has_vars = [ var in out.output_0d for out in df.output ]
    @assert all(has_vars)

    all_ns   = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df.output ]

    ## setup pane
    ax = Axis(fig[1,1]; xlabel=L"t", ylabel)

    trafo = identity
    if use_log
        trafo = (v -> isinf(v) ? NaN : v) ∘ log10 ∘ abs
    end
    norm = if use_norm_t0
        v -> v[1]
    else
        v -> 1.0
    end

    vec_ts, vec_vars = get_output_0d(df, var)

    if use_diff
        for i = 1:length(all_ns)-1
            t1, t2 = vec_ts[i], vec_ts[i+1]
            y1, y2 = vec_vars[i], vec_vars[i+1]

            ys_intrp, t_comm = GWanalysis.interpolate_common_grid([t1,t2], [y1,y2],
                                                                  idx_use_as_ref=2)
            absdiff_y = abs.(ys_intrp[2] .- ys_intrp[1])

            n1, n2 = all_ns[i], all_ns[i+1]
            lbl = L"\Delta n = %$n2 - %$n1"
            lines!(ax, t_comm, trafo.(absdiff_y/M); label=lbl)
        end
    else
        for (i,(t, var)) in enumerate(zip(vec_ts, vec_vars))
            n = all_ns[i]
            lbl = L"n = %$n"
            lines!(ax, t, trafo.(var./norm(var)); label=lbl)
        end
    end

    return ax

end


function plot_output_sphere!(fig, df, var, radius, ylabel, M;
        use_log=false, use_diff=false, use_norm_t0=false)

    has_vars = [ var in out.output_spheres for out in df.output ]
    @assert all(has_vars)

    all_ns   = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df.output ]

    ## setup pane
    ax = Axis(fig[1,1]; xlabel=L"t", ylabel)

    trafo = identity
    if use_log
        trafo = (v -> isinf(v) ? NaN : v) ∘ log10 ∘ abs
    end
    norm = if use_norm_t0
        v -> v[1]
    else
        v -> 1.0
    end

    vec_rs, vec_ts, vec_vars = GWanalysis.get_output_spheres2(df, var)

    if use_diff
        for i = 1:length(all_ns)-1
            rs1, rs2 = vec_rs[i], vec_rs[i+1]
            t1, t2 = vec_ts[i], vec_ts[i+1]
            ys1, ys2 = vec_vars[i], vec_vars[i+1]

            i1 = findfirst(==(radius), rs1) |> something
            i2 = findfirst(==(radius), rs2) |> something

            ys_intrp, t_comm = GWanalysis.interpolate_common_grid([t1,t2], [ys1[i1],ys2[i2]],
                                                                  idx_use_as_ref=2)
            absdiff_y = abs.(ys_intrp[2] .- ys_intrp[1])

            n1, n2 = all_ns[i], all_ns[i+1]
            lbl = L"\Delta n = %$n2 - %$n1"
            lines!(ax, t_comm, trafo.(absdiff_y/M); label=lbl)
        end
    else
        for i = 1:length(all_ns)
            rs, t, ys = vec_rs[i], vec_ts[i], vec_vars[i]

            ii = findfirst(==(radius), rs) |> something

            n = all_ns[i]
            lbl = L"n = %$n"
            lines!(ax, t, ys[ii]./norm(ys[ii]); label=lbl)
        end
    end

    return ax

end


let

radius = 720
M_BBS = 0.79
plt_w, plt_h = 1200, 1800

outputs = find_runs(OUTPUTS["comparison8/cts_helfer_v_0.05"])
df = load_runs(outputs)

vec_ts, AmpPsis = get_output_0d(df, "max/ana.AmpPsi")
_, idx_max = findmax(AmpPsis[end])
t_max = vec_ts[end][idx_max]


### Plot time evolution of 0d and sphere output normalized wrt initial data value

fig = Figure()
ga = fig[1,1] = GridLayout()

ax_N = plot_output_0d!(ga[1,1], df, "integral/ana.noetherQintegrand",
                       L"\frac{N(t)}{N(0)}", M_BBS,
                       use_log=false, use_diff=false, use_norm_t0=true)
ax_Madm = plot_output_sphere!(ga[2,1], df, "ana.admMintegrand", radius,
                              L"\frac{M_{ADM}(t)}{M_{ADM}(0)}", M_BBS,
                              use_log=false, use_diff=false, use_norm_t0=true)
ax_M = plot_output_0d!(ga[3,1], df, "integral/ana.totalMintegrand",
                       L"\frac{M(t)}{M(0)}", M_BBS,
                       use_log=false, use_diff=false, use_norm_t0=true)

vlines!(ax_N, t_max, color=:black, linestyle=:dash)
vlines!(ax_Madm, t_max, color=:black, linestyle=:dash)
vlines!(ax_M, t_max, color=:black, linestyle=:dash)

hidexdecorations!(ax_N, ticks=false, grid=false, minorgrid=false, minorticks=false)
hidexdecorations!(ax_Madm, ticks=false, grid=false, minorgrid=false, minorticks=false)
linkyaxes!(ax_N, ax_M, ax_Madm)
linkxaxes!(ax_N, ax_M, ax_Madm)

Legend(ga[0,1], ax_N, nbanks=2, labelvalign=:top, tellwidth=false, width=nothing,
       orientation=:horizontal, labelsize=30)

rowgap!(ga, 0)


### Plot self convergence study

ga = fig[2,1] = GridLayout()

ax_N = plot_output_0d!(ga[1,1], df, "integral/ana.noetherQintegrand",
                       L"\log_{10} |\frac{\Delta N}{M_{\textrm{BBS}}}|", M_BBS,
                       use_log=true, use_diff=true, use_norm_t0=false)
ax_Madm = plot_output_sphere!(ga[2,1], df, "ana.admMintegrand", radius,
                              L"\log_{10} |\frac{\Delta M_{ADM}}{M_{\textrm{BBS}}}|", M_BBS,
                              use_log=true, use_diff=true, use_norm_t0=false)
ax_M = plot_output_0d!(ga[3,1], df, "integral/ana.totalMintegrand",
                       L"\log_{10} |\frac{\Delta M}{M_{\textrm{BBS}}}|", M_BBS,
                       use_log=true, use_diff=true, use_norm_t0=false)

vlines!(ax_N, t_max, color=:black, linestyle=:dash)
vlines!(ax_Madm, t_max, color=:black, linestyle=:dash)
vlines!(ax_M, t_max, color=:black, linestyle=:dash)

hidexdecorations!(ax_N, ticks=false, grid=false, minorgrid=false, minorticks=false)
hidexdecorations!(ax_Madm, ticks=false, grid=false, minorgrid=false, minorticks=false)
linkyaxes!(ax_N, ax_M, ax_Madm)
linkxaxes!(ax_N, ax_M, ax_Madm)

ax_N.ylabelpadding[]    = 30
ax_M.ylabelpadding[]    = 30
ax_Madm.ylabelpadding[] = 30

Legend(ga[0,1], ax_N, nbanks=2, labelvalign=:top, tellwidth=false, width=nothing,
       orientation=:horizontal, labelsize=30)

rowgap!(ga, 0)

display(fig)

plotname = joinpath(@__DIR__, "comparison8_cts_helfer_v_0.05_N_M_Madm_combined.v2$(EXT)")
@info "saving '$plotname'"
save(plotname, fig, resolution=(plt_w, plt_h), backend=CairoMakie)

end
