using GWanalysis
using DataFrames
using GLMakie
using CairoMakie
using Printf
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")


#######################################################################
#                         Plotting functions                          #
#######################################################################


trafo(v) = log10(abs(isinf(v) ? NaN : v))


function plot_cmon(fig, df)
    ax = Axis(fig)
    get_output_0d(df, "integral/ana.Cmon")
    get_output_0d(df, "max/ana.AmpPsi")
    ns, ts, Cmons = df.n, df."t_integral/ana.Cmon", df."integral/ana.Cmon"
    for (n,t,Cmon) in zip(ns, ts, Cmons)
        label = L"n = %$n"
        lines!(ax, t, trafo.(Cmon), label=label)
    end
    # plot merger time
    ts_AmpPsi, AmpPsis = df."t_max/ana.AmpPsi", df."max/ana.AmpPsi"
    max_idx = something(findmax(AmpPsis[end]))[2]
    display(ts_AmpPsi[end][max_idx])
    vlines!(ax, ts_AmpPsi[end][max_idx], color=:black, linestyle=:dash)
    ax
end


# hack to count number of lables appearing in an axis
# shall we upstream?
function count_labels(ax::Axis)
    n = 0
    scene = ax.blockscene.children
    for s in scene
        !hasproperty(s, :plots) && continue
        for plt in s.plots
            if hasproperty(plt, :label)
                n += 1
            end
        end
    end
    n
end


function make_plot(vs, runs1, runs2, label1, label2, ylims, yticks)

    @assert length(runs1) == length(runs2) == length(vs)

    fig = Figure()
    gl = fig[1,1] = GridLayout()
    axs = Matrix{Any}(undef, (length(vs), 2))
    for (i, (df1, df2)) in enumerate(zip(runs1, runs2))
        df1.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df1.output ]
        df2.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df2.output ]
        sort!(df1, :n)
        sort!(df2, :n)
        ax1 = plot_cmon(gl[i,1], df1)
        ax2 = plot_cmon(gl[i,2], df2)
        axs[i,1] = ax1
        axs[i,2] = ax2
    end

    # insert overlapping label with boost parameter
    for i = 1:size(axs,1)
        gg = GridLayout(gl[i,1:2], tellwidth = false, tellheight = false,
                        halign = :center, valign = :top)
        box = Box(gg[1, 1], color = :white, strokecolor = :black)
        v = vs[i]
        lbl = Label(gg[1, 1], L"v = %$v", padding = (30, 30, 20, 20))
        # bring box and label in front of axis ticks
        translate!(box.blockscene, (0,0,100))
        translate!(lbl.blockscene, (0,0,100))
    end


    # hide axis decorations
    for i = 1:size(axs,1)-1
        hidexdecorations!(axs[i,1], ticks=false, grid=false, minorgrid=false)
        hidexdecorations!(axs[i,2], ticks=false, grid=false, minorgrid=false)
    end
    for i = 1:size(axs,1)
        hideydecorations!(axs[i,2], ticks=false, minorticks=false, grid=false, minorgrid=false)
    end

    for ax in axs
        ylims!(ax, ylims)
        ax.yticks[] = yticks
    end
    linkxaxes!(axs...)

    box = Box(gl[0, 1], color = :white, strokecolor = :black)
    lbl = Label(gl[0, 1], label1, padding = (10, 10, 10, 10), tellwidth=false)
    box = Box(gl[0, 2], color = :white, strokecolor = :black)
    lbl = Label(gl[0, 2], label2, padding = (10, 10, 10, 10), tellwidth=false)


    # use axis that contains the most labels to the Legend
    vaxs = axs[:]
    sort!(vaxs, by=count_labels, rev=true)
    Legend(gl[-1,:], first(vaxs), colgap=40,
           orientation=:horizontal, width=nothing, tellwidth=false)

    rowgap!(gl, 0)
    colgap!(gl, 0)

    Label(gl[1:end,0], L"\log_{10}~|\mathcal{C}_{\textrm{mon}}|", rotation=pi/2)
    Label(gl[size(axs,1)+1,1], L"t", tellwidth=false)
    Label(gl[size(axs,1)+1,2], L"t", tellwidth=false)

    fig

end


#######################################################################
#                                Plot                                 #
#######################################################################


GLMakie.closeall()

let

ylims = (-13, 3)
yticks = -12:4:2

# cve_outputs    = fuzzyfind([#r"comparison4/helfer_v_0.0$",
#                             r"comparison4/helfer_v_0.05$",
#                             r"comparison4/helfer_v_0.1$",
#                             r"comparison4/helfer_v_0.15$",
#                             r"comparison4/helfer_v_0.2$"])
# cts_outputs = fuzzyfind([#r"comparison4/cts_helfer_v_0.0$",
#                          r"comparison4/cts_helfer_v_0.05$",
#                          r"comparison4/cts_helfer_v_0.1$",
#                          r"comparison4/cts_helfer_v_0.15$",
#                          r"comparison4/cts_helfer_v_0.2$"])

cve_outputs = [# OUTPUTS["comparison4/helfer_v_0.0"],
               OUTPUTS["comparison4/helfer_v_0.05"],
               OUTPUTS["comparison4/helfer_v_0.1"],
               OUTPUTS["comparison4/helfer_v_0.15"],
               OUTPUTS["comparison4/helfer_v_0.2"]]
cts_outputs = [# OUTPUTS["comparison4/cts_helfer_v_0.0"],
               OUTPUTS["comparison4/cts_helfer_v_0.05"],
               OUTPUTS["comparison4/cts_helfer_v_0.1"],
               OUTPUTS["comparison4/cts_helfer_v_0.15"],
               OUTPUTS["comparison4/cts_helfer_v_0.2"]]

cve_outputs = find_runs.(cve_outputs)
cts_outputs = find_runs.(cts_outputs)

cve_runs = load_runs.(cve_outputs)
cts_runs = load_runs.(cts_outputs)

vs = [ #=0.0,=# 0.05, 0.1, 0.15, 0.2 ]
plotname = joinpath(@__DIR__, "cmon_comparison_cve_vs_cts+cve$(EXT)")

plt_w, plt_h = 2400, length(vs)*320

fig = make_plot(vs, cve_runs, cts_runs, "CVE", "CTS+CVE", ylims, yticks)
display(GLMakie.Screen(), fig)

@info "Saving plot to $plotname"
save(plotname, fig, resolution=(plt_w,plt_h), backend=CairoMakie)


end


let

ylims = (-13, 3)
yticks = -12:4:2

ssp_outputs = [# OUTPUTS["comparison4/naive_v_0.0"],
               OUTPUTS["comparison4/naive_v_0.05"],
               OUTPUTS["comparison4/naive_v_0.1"],
               OUTPUTS["comparison4/naive_v_0.15"],
               OUTPUTS["comparison4/naive_v_0.2"]]
cts_outputs = [# OUTPUTS["comparison4/cts_naive_v_0.0"],
               OUTPUTS["comparison4/cts_naive_v_0.05"],
               OUTPUTS["comparison4/cts_naive_v_0.1"],
               OUTPUTS["comparison4/cts_naive_v_0.15"],
               OUTPUTS["comparison4/cts_naive_v_0.2"]]

ssp_outputs = find_runs.(ssp_outputs)
cts_outputs = find_runs.(cts_outputs)

ssp_runs = load_runs.(ssp_outputs)
cts_runs = load_runs.(cts_outputs)

vs = [ #=0.0,=# 0.05, 0.1, 0.15, 0.2 ]
plotname = joinpath(@__DIR__, "cmon_comparison_ssp_vs_cts+ssp$(EXT)")

plt_w, plt_h = 2400, length(vs)*320


fig = make_plot(vs, ssp_runs, cts_runs, "SSP", "CTS+SSP", ylims, yticks)
display(GLMakie.Screen(), fig)

@info "Saving plot to $plotname"
save(plotname, fig, resolution=(plt_w,plt_h), backend=CairoMakie)


end
