using GWanalysis
using DataFrames
using GLMakie
using CairoMakie
using Printf
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")


#######################################################################
#                         Plotting functions                          #
#######################################################################


trafo(v) = log10(abs(isinf(v) ? NaN : v))


function plot_cmon_diff(fig, df)
    ax = Axis(fig)
    get_output_0d(df, "integral/ana.Cmon")
    get_output_0d(df, "max/ana.AmpPsi")
    ns, ts, Cmons = df.n, df."t_integral/ana.Cmon", df."integral/ana.Cmon"
    for i = 1:length(ns)-1
        n1, n2 = ns[i], ns[i+1]
        t1, t2 = ts[i], ts[i+1]
        Cmon1, Cmon2 = Cmons[i], Cmons[i+1]
        Cmons_intrp, t_comm = GWanalysis.interpolate_common_grid([t1,t2], [Cmon1,Cmon2],
                                                                 idx_use_as_ref=2)
        diff = Cmons_intrp[2] .- Cmons_intrp[1]
        label = L"\Delta n = %$n2 - %$n1"
        lines!(ax, t_comm, trafo.(diff), label=label)
    end
    # plot merger time
    ts_AmpPsi, AmpPsis = df."t_max/ana.AmpPsi", df."max/ana.AmpPsi"
    max_idx = something(findmax(AmpPsis[end]))[2]
    vlines!(ax, ts_AmpPsi[end][max_idx], color=:black, linestyle=:dash)
    ax
end


# hack to count number of lables appearing in an axis
# shall we upstream?
function count_labels(ax::Axis)
    n = 0
    scene = ax.blockscene.children
    for s in scene
        !hasproperty(s, :plots) && continue
        for plt in s.plots
            if hasproperty(plt, :label)
                n += 1
            end
        end
    end
    n
end


function make_plot(vs, runs1, ylims, yticks)

    @assert length(runs1) == length(vs)

    fig = Figure()
    gl = fig[1,1] = GridLayout()
    axs = Matrix{Any}(undef, (length(vs), 1))
    for (i, df1) in enumerate(runs1)
        df1.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df1.output ]
        sort!(df1, :n)
        ax1 = plot_cmon_diff(gl[i,1], df1)
        axs[i,1] = ax1
    end

    # insert overlapping label with boost parameter
    for i = 1:size(axs,1)
        gg = GridLayout(gl[i,1], tellwidth = false, tellheight = false,
                        halign = :right, valign = :top)
        box = Box(gg[1, 1], color = :white, strokecolor = :black)
        v = vs[i]
        lbl = Label(gg[1, 1], L"v = %$v", padding = (30, 30, 20, 20))
        # bring box and label in front of axis ticks
        translate!(box.blockscene, (0,0,100))
        translate!(lbl.blockscene, (0,0,100))
    end


    # hide axis decorations
    for i = 1:size(axs,1)-1
        hidexdecorations!(axs[i,1], ticks=false, grid=false, minorgrid=false)
    end

    for ax in axs
        ylims!(ax, ylims)
        ax.yticks[] = yticks
    end
    linkxaxes!(axs...)

    # use axis that contains the most labels to the Legend
    vaxs = axs[:]
    sort!(vaxs, by=count_labels, rev=true)
    Legend(gl[0,:], first(vaxs), colgap=40, nbanks=2,
           orientation=:horizontal, width=nothing, tellwidth=false)

    rowgap!(gl, 0)
    colgap!(gl, 0)

    Label(gl[1:end,0], L"\log_{10}~|\Delta \mathcal{C}_{\textrm{mon}}|", rotation=pi/2)
    Label(gl[size(axs,1)+1,1], L"t", tellwidth=false)

    fig

end


#######################################################################
#                                Plot                                 #
#######################################################################

GLMakie.closeall()

let


ylims = (-15.5, 3.5)
yticks = -14:4:2

cve_outputs = [# OUTPUTS["comparison4/helfer_v_0.0"],
               OUTPUTS["comparison4/helfer_v_0.05"],
               OUTPUTS["comparison4/helfer_v_0.1"],
               OUTPUTS["comparison4/helfer_v_0.15"],
               OUTPUTS["comparison4/helfer_v_0.2"]]
cve_outputs = find_runs.(cve_outputs)
cve_runs = load_runs.(cve_outputs)

vs = [ #=0.0,=# 0.05, 0.1, 0.15, 0.2 ]
plotname = joinpath(@__DIR__, "diffs_cmon_comparison_cve$(EXT)")

plt_w, plt_h = 1200, length(vs)*300

fig = make_plot(vs, cve_runs, ylims, yticks)
display(GLMakie.Screen(), fig)

@info "Saving plot to $plotname"
save(plotname, fig, resolution=(plt_w,plt_h), backend=CairoMakie)


end


let

ylims = (-15.5, 3.5)
yticks = -14:4:2

ssp_outputs = [# OUTPUTS["comparison4/naive_v_0.0"],
               OUTPUTS["comparison4/naive_v_0.05"],
               OUTPUTS["comparison4/naive_v_0.1"],
               OUTPUTS["comparison4/naive_v_0.15"],
               OUTPUTS["comparison4/naive_v_0.2"]]
ssp_outputs = find_runs.(ssp_outputs)
ssp_runs = load_runs.(ssp_outputs)
vs = [ #=0.0,=# 0.05, 0.1, 0.15, 0.2 ]
plotname = joinpath(@__DIR__, "diffs_cmon_comparison_ssp$(EXT)")

plt_w, plt_h = 1200, length(vs)*300

fig = make_plot(vs, ssp_runs, ylims, yticks)
display(GLMakie.Screen(), fig)

@info "Saving plot to $plotname"
save(plotname, fig, resolution=(plt_w,plt_h), backend=CairoMakie)


end
