using GWanalysis
using DataFrames
using GLMakie
using Printf
using CairoMakie
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")



function extract_data(groups, n #= resolutin =#)
    vec_Es = Vector{Float64}[]
    vec_avg_Es = Vector{Float64}[]
    vec_exmin_Es = Vector{Float64}[]
    vec_exmax_Es = Vector{Float64}[]
    for group in groups

        runs = vcat(group...)
        runs.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in runs.output ]
        runs = filter(eachrow(runs)) do row
            row.n == n
        end |> DataFrame

        # results with three cutoff frequencies 1/2R, 1/R, 2/R to account for systematics
        GWanalysis.get_output_analysis(runs, "E_Psi4_20_fcut1")
        GWanalysis.get_output_analysis(runs, "E_Psi4_20_fcut2")
        GWanalysis.get_output_analysis(runs, "E_Psi4_20_fcut3")

        vec_E1s = runs."E_Psi4_20_fcut1"
        vec_E2s = runs."E_Psi4_20_fcut2"
        vec_E3s = runs."E_Psi4_20_fcut3"
        # extract total energy (last entry) for each run and each extrapolation order
        vec_E1end = [ [ E[end] for E in Es] for Es in vec_E1s ]
        vec_E2end = [ [ E[end] for E in Es] for Es in vec_E2s ]
        vec_E3end = [ [ E[end] for E in Es] for Es in vec_E3s ]

        # for each run, average over all cutoffs and extrapolation orders
        Es = []
        avg_Es = []
        exmin_Es = []
        exmax_Es = []
        for n = 1:nrow(runs)
            E1, E2, E3 = vec_E1end[n], vec_E2end[n], vec_E3end[n]
            sum_E1, sum_E2, sum_E3 = sum(E1), sum(E2), sum(E3)
            avg_E = (sum_E1 + sum_E2 + sum_E3) / (length(E1) + length(E2) + length(E3))
            exmin, exmax = extrema(vcat(E1, E2, E3))
            push!(avg_Es, avg_E)
            push!(exmin_Es, exmin)
            push!(exmax_Es, exmax)
            append!(Es, E1)
            append!(Es, E2)
            append!(Es, E3)
        end

        push!(vec_Es, Es)
        push!(vec_avg_Es, avg_Es)
        push!(vec_exmax_Es, exmax_Es)
        push!(vec_exmin_Es, exmin_Es)

    end

    return vec_Es, vec_avg_Es, vec_exmin_Es, vec_exmax_Es
end


#######################################################################
#                               Script                                #
#######################################################################


let

M    = 0.79
nref = 13 # polynomial order for finite resolution error estimation
n    = 15 # polynomial order

plotname = "comparison8_energy_vs_v$(EXT)"
out_group0 = [ OUTPUTS["comparison8/cts_helfer_v_0.0"],
               OUTPUTS["comparison8/helfer_v_0.0"],
               OUTPUTS["comparison8/cts_naive_v_0.0"],
               OUTPUTS["comparison8/naive_v_0.0"] ]
out_group1 = [ OUTPUTS["comparison8/cts_helfer_v_0.05"],
               OUTPUTS["comparison8/helfer_v_0.05"],
               OUTPUTS["comparison8/cts_naive_v_0.05"],
               OUTPUTS["comparison8/naive_v_0.05"] ]
out_group2 = [ OUTPUTS["comparison8/cts_helfer_v_0.1"],
               OUTPUTS["comparison8/helfer_v_0.1"],
               OUTPUTS["comparison8/cts_naive_v_0.1"],
               OUTPUTS["comparison8/naive_v_0.1"] ]
out_group3 = [ OUTPUTS["comparison8/cts_helfer_v_0.15"],
               OUTPUTS["comparison8/helfer_v_0.15"],
               OUTPUTS["comparison8/cts_naive_v_0.15"],
               OUTPUTS["comparison8/naive_v_0.15"] ]
out_group4 = [ OUTPUTS["comparison8/cts_helfer_v_0.2"],
               OUTPUTS["comparison8/helfer_v_0.2"],
               OUTPUTS["comparison8/cts_naive_v_0.2"],
               OUTPUTS["comparison8/naive_v_0.2"] ]

group0 = find_runs.(out_group0)
group1 = find_runs.(out_group1)
group2 = find_runs.(out_group2)
group3 = find_runs.(out_group3)
group4 = find_runs.(out_group4)
groups = [group0, group1, group2, group3, group4]
df_groups = [ load_runs.(g) for g in groups ]
vs = [ 0.0, 0.05, 0.1, 0.15, 0.2 ]
labels = [ "CTS+CVE", "CVE", "CTS+SSP", "SSP" ]
markers = [ :rect, :star4, :diamond, :circle ]
markersize = [ 35, 25, 35, 20 ]
linestyles = [ :solid, :dash, :solid, :dash ]


vec_Es, vec_avg_Es, vec_exmin_Es, vec_exmax_Es                 = extract_data(df_groups, n)
vec_ref_Es, vec_ref_avg_Es, vec_ref_exmin_Es, vec_ref_exmax_Es = extract_data(df_groups, nref)


grouped_avg_Es       = vcat(vec_avg_Es'...)
grouped_exmin_Es     = vcat(vec_exmin_Es'...)
grouped_exmax_Es     = vcat(vec_exmax_Es'...)
grouped_ref_avg_Es   = vcat(vec_ref_avg_Es'...)
grouped_ref_exmin_Es = vcat(vec_ref_exmin_Es'...)
grouped_ref_exmax_Es = vcat(vec_ref_exmax_Es'...)

# finite resolution error is taken to be the biggest difference between energies
# for a specific fcut and Nextrp and fixed boost
diffs_vec_Es = [ abs.(Es1 .- Es2) for (Es1, Es2) in zip(vec_Es, vec_ref_Es) ]
finite_res_errs = [ maximum(d) for d in diffs_vec_Es  ]

# the systematic error is taken to be the biggest difference between a data point and the
# mean of the data
# fcut_errs = [ max(abs(grouped_avg_Es[ij-grouped_exmin_Es[ij])),abs(grouped_avg_Es[ij]-grouped_ref_exmax_Es[ij]))
#               for ij in eachindex(IndexCartesian(), grouped_exmin_Es, grouped_exmax_Es) ]
fcut_errs = zeros((length(finite_res_errs),length(labels)))
for ij in eachindex(IndexCartesian(), grouped_avg_Es, grouped_exmin_Es, grouped_exmax_Es)
    E, Emin, Emax = grouped_avg_Es[ij], grouped_exmin_Es[ij], grouped_ref_exmax_Es[ij]
    fcut_errs[ij] = max(abs(E-Emin),abs(E-Emax))
end

# add errors in quadrature
errs = @. sqrt(finite_res_errs^2 + fcut_errs^2)
display(errs)

### print differences between the two resolutions
# display(diffs_vec_Es)
# display([ extrema(d) for d in diffs_vec_Es])
# display([ abs.(a1 .- a2) for (a1,a2) in zip(grouped_avg_Es, grouped_ref_avg_Es) ])
# display([ abs.(e1 .- e2) for (e1,e2) in zip(grouped_exmin_Es, grouped_ref_exmin_Es) ])
# display([ abs.(e1 .- e2) for (e1,e2) in zip(grouped_exmax_Es, grouped_ref_exmax_Es) ])


fig = Figure()
ga = fig[1,1] = GridLayout()
ax = Axis(ga[1,1], xlabel=L"|~v~|", ylabel=L"E/M_{BBS}~[%]")

### plot the 'ref' data in gray in the background
# for i = 1:length(labels)
#     Es       = grouped_ref_avg_Es[:,i]
#     exmin_Es = grouped_ref_exmin_Es[:,i]
#     exmax_Es = grouped_ref_exmax_Es[:,i]
#     errorbars!(ax, vs, Es./M*100, (Es.-exmin_Es)./M*100, (exmax_Es-Es)./M*100,
#                whiskerwidth=40, linewidth=1, color=:gray)
#     scatterlines!(ax, vs, Es./M*100, label=labels[i], markersize=markersize[i], linestyle=linestyles[i],
#                   linewidth=2, color=:gray, marker=markers[i])
# end


for i = 1:length(labels)
    Es = grouped_avg_Es[:,i]
    err = errs[:,i]
    c = Makie.wong_colors()[i]
    errorbars!(ax, vs, Es./M*100, err./M*100, err./M*100,
               whiskerwidth=60, linewidth=4, color=c)
    scatterlines!(ax, vs, Es./M*100, label=labels[i], markersize=markersize[i], linestyle=linestyles[i],
                  linewidth=5, color=c, marker=markers[i])
    # band!(ax, vs, exmin_Es./M*100, exmax_Es./M*100,
    #       color=RGBAf(c.r,c.g,c.b,0.2))
end
ylims!(ax, (0.002, 0.032))
Legend(ga[0,1], ax, width=nothing, tellheight=true, orientation=:horizontal,
       colgap=30, patchsize=(70,20))
rowgap!(ga, 0)

filename = joinpath(@__DIR__, plotname)
@info "saving plot to $filename..."
save(filename, fig; resolution=(1200,800), backend=CairoMakie)

display(fig)

end
