using GWanalysis
using DelimitedFiles
using DataFrames
using FFTW
using Printf


include("../rootdirs.jl")


#######################################################################
#                              Functions                              #
#######################################################################


log_trafo = (v -> isinf(v) ? NaN : v) ∘ log10 ∘ abs


function integral_factor(f, fcut)
    if abs(f) < fcut
        g = 2 * (f/fcut)^3 - 3 * (f/fcut)^2 + 1
        eps = 1e-10
        return f / (f^2 + g * eps) / (im*2*pi)
    else
        return 1/(im*2*pi*f)
    end
end


function dEdt_20(Ns, u, win_Res, filter_N, filter_fcut; zeropad=false, apply_butterworth=true)

    rng = 1:length(u)
    if zeropad
        Res = [ GWanalysis.zeropad(Re, length(u))[1] for Re in Res ]
        u, rng = linearpad(u, length(u))
    end

    # fourier transform
    du = u[2]-u[1]
    fts = rfft.(win_Res)
    f = rfftfreq(length(u), 1/du)

    fts = rfft.(win_Res)

    # high pass filter
    filter_fts = if apply_butterworth
        H = GWanalysis.butterworth_transfer_fn.(f, filter_fcut, filter_N; type=:high)
        [ ft .* abs.(H) for ft in fts ]
    else
        fts
    end

    # fixed frequency integration to find dE/dt
    int_factor = [ abs(fi) < filter_fcut ? 1/(im*2*pi*filter_fcut) : 1/(im*2*pi*fi) for fi in f ]
    # int_factor = integral_factor.(f, filter_fcut)
    integrands_dhdt = [ int_factor .* ft for ft in filter_fts ]
    dhdts = [ irfft(int, length(u))[rng] for int in integrands_dhdt ]
    dEdts = [ dhdt.^2 / (16*pi) for dhdt in dhdts ]
    integrands_h = [ int_factor.^2 .* ft for ft in filter_fts ]
    hs = [ irfft(int, length(u))[rng] for int in integrands_h ]

    # filter_Res = [ irfft(ft, length(u)) for ft in filter_fts ]
    # dEdts = deepcopy(filter_Res)
    # du = u[2]-u[1]
    # GWanalysis.integrate_trapezoidal!.(du, dEdts)
    # for i = 1:length(dEdts)
    #     dEdts[i] = dEdts[i].^2 / (16*pi)
    # end

    # trapezoidal integation to find E
    Es = deepcopy(dEdts)
    GWanalysis.integrate_trapezoidal!.(du, Es)


    return dEdts, Es, dhdts, hs

end


function compute_dEdt_E(df, M, filter_N, filter_fcut, win_umin, win_umax, win_slope,
        filename_suffix)

    # window signals
    window(u) = GWanalysis.window_tanh(u, win_umin, win_slope) * GWanalysis.window_tanh(u, win_umax, -win_slope)

    for row in eachrow(df)
        u    =  row."Psi4_20_Nextrp_1:Time"
        Res = [ row."Psi4_20_Nextrp_1:Psi4",
                row."Psi4_20_Nextrp_2:Psi4",
                row."Psi4_20_Nextrp_3:Psi4",
                row."Psi4_20_Nextrp_4:Psi4",
                row."Psi4_20_Nextrp_5:Psi4" ]
        Ns   = [ 1, 2, 3, 4, 5 ]
        win_Res = [ window.(u) .* Re for Re in Res ]
        dEdts, Es, dhdts, hs = dEdt_20(Ns, u, win_Res, filter_N, filter_fcut, zeropad=false)
        Es_end = reduce(vcat, [ E[end] for E in Es ])

        # save data
        analysis_dir = joinpath(row.output.dir, "analysis")
        !isdir(analysis_dir) && mkdir(analysis_dir)

        filename = joinpath(analysis_dir, "win_Psi4_20$filename_suffix")
        @info "writing windowed Psi4_20 output to $filename"
        open(filename, "w") do file
            println(file, "# Description: windowed Psi4_20 output for computation of radidated energy")
            println(file, "# Window: umin = $win_umin, umax = $win_umax, slope = $win_slope, type = tanh")
            print(file, "# Time")
            for N in Ns
                @printf(file, " N=%.d", N)
            end
            println(file)
            data = hcat( u, win_Res... )
            writedlm(file, data)
        end

        filename = joinpath(analysis_dir, "dEdt_Psi4_20$filename_suffix")
        @info "writing radiated energy flux to $filename"
        open(filename, "w") do file
            println(file, "# Description: radiated energy flux dE/dt from Psi4_20")
            println(file, "# Filter: fcut = $filter_fcut, N = $filter_N")
            println(file, "# Window: umin = $win_umin, umax = $win_umax, slope = $win_slope, type = tanh")
            print(file, "# Time")
            for N in Ns
                @printf(file, " N=%.d", N)
            end
            println(file)
            data = hcat( u, dEdts... )
            writedlm(file, data)
        end

        filename = joinpath(analysis_dir, "E_Psi4_20$filename_suffix")
        @info "writing radiated energy to $filename"
        open(filename, "w") do file
            println(file, "# Description: radiated energy E from Psi4_20")
            println(file, "# Filter: fcut = $filter_fcut, N = $filter_N")
            println(file, "# Window: umin = $win_umin, umax = $win_umax, slope = $win_slope, type = tanh")
            print(file, "# Time")
            for N in Ns
                @printf(file, " N=%.d", N)
            end
            println(file)
            data = hcat( u, Es... )
            writedlm(file, data)
        end

        filename = joinpath(analysis_dir, "dhdt_Psi4_20$filename_suffix")
        @info "writing radiated energy to $filename"
        open(filename, "w") do file
            println(file, "# Description: 'strain flux' from Psi4_20")
            println(file, "# Filter: fcut = $filter_fcut, N = $filter_N")
            println(file, "# Window: umin = $win_umin, umax = $win_umax, slope = $win_slope, type = tanh")
            print(file, "# Time")
            for N in Ns
                @printf(file, " N=%.d", N)
            end
            println(file)
            data = hcat( u, dhdts... )
            writedlm(file, data)
        end

        filename = joinpath(analysis_dir, "h_Psi4_20$filename_suffix")
        @info "writing radiated energy to $filename"
        open(filename, "w") do file
            println(file, "# Description: strain from Psi4_20")
            println(file, "# Filter: fcut = $filter_fcut, N = $filter_N")
            println(file, "# Window: umin = $win_umin, umax = $win_umax, slope = $win_slope, type = tanh")
            print(file, "# Time")
            for N in Ns
                @printf(file, " N=%.d", N)
            end
            println(file)
            data = hcat( u, hs... )
            writedlm(file, data)
        end


        # redo reconstruction without butterworth filter to gauge its effect
        dEdts, Es, dhdts, hs = dEdt_20(Ns, u, win_Res, filter_N, filter_fcut, zeropad=false,
                                      apply_butterworth=false)

        filename = joinpath(analysis_dir, "no_butterworth_dEdt_Psi4_20$(filename_suffix)")
        @info "writing radiated energy flux to $filename"
        open(filename, "w") do file
            println(file, "# Description: radiated energy flux dE/dt from Psi4_20")
            println(file, "# Filter: fcut = $filter_fcut")
            println(file, "# Window: umin = $win_umin, umax = $win_umax, slope = $win_slope, type = tanh")
            print(file, "# Time")
            for N in Ns
                @printf(file, " N=%.d", N)
            end
            println(file)
            data = hcat( u, dEdts... )
            writedlm(file, data)
        end

        filename = joinpath(analysis_dir, "no_butterworth_dhdt_Psi4_20$filename_suffix")
        @info "writing radiated energy to $filename"
        open(filename, "w") do file
            println(file, "# Description: 'strain flux' from Psi4_20")
            println(file, "# Filter: fcut = $filter_fcut")
            println(file, "# Window: umin = $win_umin, umax = $win_umax, slope = $win_slope, type = tanh")
            print(file, "# Time")
            for N in Ns
                @printf(file, " N=%.d", N)
            end
            println(file)
            data = hcat( u, dhdts... )
            writedlm(file, data)
        end

        filename = joinpath(analysis_dir, "no_butterworth_h_Psi4_20$filename_suffix")
        @info "writing radiated energy to $filename"
        open(filename, "w") do file
            println(file, "# Description: strain from Psi4_20")
            println(file, "# Filter: fcut = $filter_fcut")
            println(file, "# Window: umin = $win_umin, umax = $win_umax, slope = $win_slope, type = tanh")
            print(file, "# Time")
            for N in Ns
                @printf(file, " N=%.d", N)
            end
            println(file)
            data = hcat( u, hs... )
            writedlm(file, data)
        end

    end

end


#######################################################################
#                               Script                                #
#######################################################################


let


M = 0.79
filter_N = 10

outputs = [ find_runs(OUTPUTS["comparison8/cts_helfer_v_0.0"]),
            find_runs(OUTPUTS["comparison8/cts_helfer_v_0.05"]),
            find_runs(OUTPUTS["comparison8/cts_helfer_v_0.1"]),
            find_runs(OUTPUTS["comparison8/cts_helfer_v_0.15"]),
            find_runs(OUTPUTS["comparison8/cts_helfer_v_0.2"]),
            find_runs(OUTPUTS["comparison8/cts_naive_v_0.0"]),
            find_runs(OUTPUTS["comparison8/cts_naive_v_0.05"]),
            find_runs(OUTPUTS["comparison8/cts_naive_v_0.1"]),
            find_runs(OUTPUTS["comparison8/cts_naive_v_0.15"]),
            find_runs(OUTPUTS["comparison8/cts_naive_v_0.2"]),
            find_runs(OUTPUTS["comparison8/helfer_v_0.0"]),
            find_runs(OUTPUTS["comparison8/helfer_v_0.05"]),
            find_runs(OUTPUTS["comparison8/helfer_v_0.1"]),
            find_runs(OUTPUTS["comparison8/helfer_v_0.15"]),
            find_runs(OUTPUTS["comparison8/helfer_v_0.2"]),
            find_runs(OUTPUTS["comparison8/naive_v_0.0"]),
            find_runs(OUTPUTS["comparison8/naive_v_0.05"]),
            find_runs(OUTPUTS["comparison8/naive_v_0.1"]),
            find_runs(OUTPUTS["comparison8/naive_v_0.15"]),
            find_runs(OUTPUTS["comparison8/naive_v_0.2"]),
]

win_umin, win_umax, win_slope = 100, 3000, 0.1
R = 800.0
fcuts = [ 2/R, 1/R, 3/R ]
@info fcuts


for outs in outputs
    df = load_runs(outs)

    GWanalysis.get_output_analysis2(df, "Psi4_20_Nextrp_1")
    GWanalysis.get_output_analysis2(df, "Psi4_20_Nextrp_2")
    GWanalysis.get_output_analysis2(df, "Psi4_20_Nextrp_3")
    GWanalysis.get_output_analysis2(df, "Psi4_20_Nextrp_4")
    GWanalysis.get_output_analysis2(df, "Psi4_20_Nextrp_5")
    df.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df.output ]
    df.R = [ GWanalysis.getparameter(Float64, out, "grid.sphere.max.x") for out in df.output ]
    @assert all(df.R .≈ R)
    @assert length(unique(df.R)) == 1

    for (i,fcut) in enumerate(fcuts)
        compute_dEdt_E(df, M, filter_N, fcut, win_umin, win_umax, win_slope, "_fcut$i")
    end

end


end
