using GWanalysis
using DataFrames
using GLMakie
using CairoMakie
using Printf
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")


#######################################################################
#                         Plotting functions                          #
#######################################################################


function find_two_extrema(u, f)
    df = diff(f)
    idx1, idx2 = 0, 0
    absf1, absf2 = 0, 0
    for i = 1:length(df)-1
        df1, df2 = df[i], df[i+1]
        if df1 * df2 < 0
            absf = abs(f[i])
            if absf > absf1
                idx2, idx1 = idx1, i
                absf2, absf1 = absf1, absf
            elseif absf > absf2
                idx2 = i
                absf2 = absf
            end
        end
    end
    @assert 0 < idx2 < idx1 <= length(f)
    return idx1, idx2
end


function moving_average(y::AbstractVector, w=40)
    avg = similar(y)
    N = length(y)
    w2 = div(w,2)
    l, r = w2+isodd(w), N-w2+iseven(w)
    # middle section
    for i in l:r
        rng = 1+i-l:i-(l+1)+w
        vy = view(y, rng)
        avg[i] = sum(vy) / w
    end
    # fill beginning and end with first and last values of middle section
    for i in 1:l-1
        avg[i] = avg[l]
    end
    for i in r+1:N
        avg[i] = avg[r]
    end
    return avg
end


function plot_psi4_20!(fig, df, radius, M)

    var = "Psi4_20"
    has_vars = [ var in out.output_spheres for out in df.output ]
    @assert all(has_vars)

    zAs = [ GWanalysis.getparameter(Float64, out, "complex_scalarfield.initialdata.binary_boson_star.star_A.center.z")
            for out in df.output ]
    zBs = [ GWanalysis.getparameter(Float64, out, "complex_scalarfield.initialdata.binary_boson_star.star_B.center.z")
            for out in df.output ]
    ds = [ abs(zA) + abs(zB) for (zA,zB) in zip(zAs, zBs) ]

    ## setup pane
    ax = Axis(fig[1,1]; xlabel=L"t/M_{\textrm{BBS}}")

    vec_rs, vec_ts, vec_Res, _ = GWanalysis.get_output_spheres_modes(df, var)

    @assert all(rs -> radius in rs, vec_rs)

    counter = 1
    for (d, rs, t, Res) in zip(ds, vec_rs, vec_ts, vec_Res)
        for (r,Re) in zip(rs,Res)
            r != radius && continue
            maxidx1, maxidx2 = find_two_extrema(t, Re)
            t1, t2 = t[maxidx1], t[maxidx2]
            dd = Int64(round(d))
            lbl = L"d = %$dd"
            c = Makie.wong_colors()[counter]
            cc = RGBAf(c.r,c.g,c.b, isodd(counter) ? 1.0 : 0.6)
            ls = isodd(counter) ? :solid : :dash
            counter += 1
            lines!(ax, t./M, M .* Re; label=lbl, linestyle=ls)
            vlines!(ax, t1./M, color=:black, linestyle=:dash)
            vlines!(ax, t2./M, color=:black, linestyle=:dash)
        end
    end

    return ax

end


function plot_diff_psi4_20!(fig, df1, df2, radius, M)

    var = "Psi4_20"

    zAs = [ GWanalysis.getparameter(Float64, out, "complex_scalarfield.initialdata.binary_boson_star.star_A.center.z")
            for out in df1.output ]
    zBs = [ GWanalysis.getparameter(Float64, out, "complex_scalarfield.initialdata.binary_boson_star.star_B.center.z")
            for out in df1.output ]
    ds = [ abs(zA) + abs(zB) for (zA,zB) in zip(zAs, zBs) ]

    ## setup pane
    ax = Axis(fig[1,1]; xlabel=L"t/M_{\textrm{BBS}}")

    vec_r1s, vec_t1s, vec_Re1s, _ = GWanalysis.get_output_spheres_modes(df1, var)
    vec_r2s, vec_t2s, vec_Re2s, _ = GWanalysis.get_output_spheres_modes(df2, var)

    @assert all(rs -> radius in rs, vec_r1s)
    @assert all(rs -> radius in rs, vec_r2s)

    trafo = (v -> isinf(v) ? NaN : v) ∘ log10 ∘ abs

    counter = 1
    for (d, r1s, t1, Re1s, r2s, t2, Re2s) in zip(ds, vec_r1s, vec_t1s, vec_Re1s, vec_r2s, vec_t2s, vec_Re2s)
        idx1 = findfirst(r -> r == radius, r1s) |> something
        idx2 = findfirst(r -> r == radius, r2s) |> something

        r1 = r1s[idx1]
        r2 = r2s[idx2]
        Re1 = Re1s[idx1]
        Re2 = Re2s[idx2]

        Res_intrp, t_intrp = GWanalysis.interpolate_common_grid([t1, t2], [ Re1, Re2 ])
        Re1_intrp, Re2_intrp = Res_intrp
        diff_Res = @. abs(Re1_intrp - Re2_intrp)
        avg_diff_Res = moving_average(diff_Res)

        dd = Int64(round(d))
        lbl = L"d = %$dd"
        c = Makie.wong_colors()[counter]
        cc = RGBAf(c.r,c.g,c.b, isodd(counter) ? 1.0 : 0.6)
        ls = isodd(counter) ? :solid : :dash
        counter += 1
        lines!(ax, t_intrp./M, trafo.(M .* avg_diff_Res); label=lbl, linestyle=ls)

        maxidx1, maxidx2 = find_two_extrema(t1, Re1)
        t1, t2 = t1[maxidx1], t1[maxidx2]
        display(t1./M)
        vlines!(ax, t1./M, color=:black, linestyle=:dash)
        vlines!(ax, t2./M, color=:black, linestyle=:dash)
    end

    return ax

end


function plot_psi4_20(out_n, out_h, out_cn, out_ch, radius, M,
        xlims, xticks, ylims, yticks)

    fig = Figure()
    ga = fig[1,1] = GridLayout()

    axs = Matrix{Any}(undef, 2,2)

    axs[1,1] = plot_psi4_20!(ga[1,1], out_cn, radius, M)
    axs[1,2] = plot_psi4_20!(ga[1,2], out_ch, radius, M)
    axs[2,1] = plot_diff_psi4_20!(ga[2,1], out_n, out_cn, radius, M)
    axs[2,2] = plot_diff_psi4_20!(ga[2,2], out_h, out_ch, radius, M)

    hidexdecorations!(axs[1,1], ticks=false, grid=false, minorgrid=false, minorticks=false)
    hidexdecorations!(axs[1,2], ticks=false, grid=false, minorgrid=false, minorticks=false)
    hideydecorations!(axs[1,2], ticks=false, grid=false, minorgrid=false, minorticks=false)
    hideydecorations!(axs[2,2], ticks=false, grid=false, minorgrid=false, minorticks=false)
    axs[1,1].xticks[] = xticks
    axs[1,2].xticks[] = xticks
    axs[2,1].xticks[] = xticks
    axs[2,2].xticks[] = xticks
    xlims!(axs[1,1], xlims)
    xlims!(axs[1,2], xlims)
    xlims!(axs[2,1], xlims)
    xlims!(axs[2,2], xlims)
    axs[1,1].yticks[] = yticks
    axs[1,2].yticks[] = yticks
    ylims!(axs[1,1], ylims)
    ylims!(axs[1,2], ylims)
    linkyaxes!(axs[2,1], axs[2,2])

    Legend(ga[0,1:2], axs[1,1], labelvalign=:top, tellwidth=false, width=nothing,
           nbanks=1, orientation=:horizontal, colgap=30)
    Label(ga[1,0], text=L"M_{\textrm{BBS}}~\text{Re}(r_{ext}\Psi_{4,20})", tellwidth=true, tellheight=false,
          rotation=pi/2, fontsize=35, padding=(0,25,0,0))
    Label(ga[2,0], text=L"\log_{10}|\Delta(M_{\textrm{BBS}}~\text{Re}(r_{ext}\Psi_{4,20}))|", tellwidth=true, tellheight=false,
          rotation=pi/2, fontsize=35, padding=(0,25,0,0))

    rowgap!(ga, 0)
    colgap!(ga, 0)

    # hack in text box,
    # cf. https://discourse.julialang.org/t/cairomakie-text-box/103783/3?u=fatteneder
    padding = (10,10,10,10)
    gl = GridLayout(ga[1,1],
                    tellwidth = false, tellheight = false,
                    halign = :right, valign = :bottom)
    box = Box(gl[1, 1], color = :white, strokecolor = :black)
    lbl = Label(gl[1, 1], "CTS+SSP", padding = padding, fontsize=30)
    translate!(box.blockscene, (0,0,100))
    translate!(lbl.blockscene, (0,0,100))
    gl = GridLayout(ga[1,2],
                    tellwidth = false, tellheight = false,
                    halign = :right, valign = :bottom)
    box = Box(gl[1, 1], color = :white, strokecolor = :black)
    lbl = Label(gl[1, 1], "CTS+CVE", padding = padding, fontsize=30)
    translate!(box.blockscene, (0,0,100))
    translate!(lbl.blockscene, (0,0,100))
    gl = GridLayout(ga[2,1],
                    tellwidth = false, tellheight = false,
                    halign = :right, valign = :bottom)
    box = Box(gl[1, 1], color = :white, strokecolor = :black)
    lbl = Label(gl[1, 1], "SSP vs. CTS+SSP", padding = padding, fontsize=30)
    translate!(box.blockscene, (0,0,100))
    translate!(lbl.blockscene, (0,0,100))
    gl = GridLayout(ga[2,2],
                    tellwidth = false, tellheight = false,
                    halign = :right, valign = :bottom)
    box = Box(gl[1, 1], color = :white, strokecolor = :black)
    lbl = Label(gl[1, 1], "CVE vs. CTS+CVE", padding = padding, fontsize=30)
    translate!(box.blockscene, (0,0,100))
    translate!(lbl.blockscene, (0,0,100))

    return fig

end


let

radius = 720
M_BBS = 2 * 0.395
xlims = (1050,1850)
xticks = 1100:100:1800
ylims = (-0.0011, 0.0008)
yticks = -0.001:0.0004:0.0006

plt_w, plt_h = 2400, 1000

plotname = joinpath(@__DIR__, "psi4_comparison9_psi4_v_d$(EXT)")
out_n  = find_runs(OUTPUTS["comparison9/naive"])
out_h  = find_runs(OUTPUTS["comparison9/helfer"])
out_cn = find_runs(OUTPUTS["comparison9/cts_naive"])
out_ch = find_runs(OUTPUTS["comparison9/cts_helfer"])

out_ch = load_runs(out_ch)
out_cn = load_runs(out_cn)
out_h  = load_runs(out_h)
out_n  = load_runs(out_n)

### compare for helfer/equal-mass-fixed superposition

fig = plot_psi4_20(out_n, out_h, out_cn, out_ch,
                   radius, M_BBS, xlims, xticks, ylims, yticks)

display(fig)

@info "saving '$plotname'"
save(plotname, fig, resolution=(plt_w, plt_h), backend=CairoMakie)

end
