using GWanalysis
using DataFrames
using GLMakie
using CairoMakie
using Printf
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")


#######################################################################
#                          Plotting functions                         #
#######################################################################


function moving_average(y::AbstractVector, w=40)
    avg = similar(y)
    N = length(y)
    w2 = div(w,2)
    l, r = w2+isodd(w), N-w2+iseven(w)
    # middle section
    for i in l:r
        rng = 1+i-l:i-(l+1)+w
        vy = view(y, rng)
        avg[i] = sum(vy) / w
    end
    # fill beginning and end with first and last values of middle section
    for i in 1:l-1
        avg[i] = avg[l]
    end
    for i in r+1:N
        avg[i] = avg[r]
    end
    return avg
end


function plot_psi4_diffs(df, M, varname, r, ylabel;
                         ylimits=nothing, xlimits=nothing,
                         yticks=nothing, xticks=nothing,
                         nbanks=1)

    df.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df.output ]
    sort!(df, :n)

    @assert allunique(df.n)

    all_vars = [ out.output_spheres for out in df.output ]
    all_vars = intersect(all_vars...)
    filter!(all_vars) do var
        startswith(var, "Psi4")
    end
    if length(all_vars) == 0
        error("no Psi4 outputs found for provided runs")
    end

    # load data
    vec_rs, vec_t, vec_Res, vec_Ims, vec_us = GWanalysis.get_output_spheres_modes(df, varname, M)
    vec_t_amppsi, vec_amppsi = GWanalysis.get_output_0d(df, "max/ana.AmpPsi")

    # find merger time from maximum of scalar field amplitude
    merger_ts = []
    for (t,AmpPsi) in zip(vec_t_amppsi, vec_amppsi)
        _, idx = findmax(AmpPsi)
        push!(merger_ts, t[idx])
    end
    merger_t = sum(merger_ts)/length(merger_ts)

    # prepare data
    trafo = (v -> isinf(v) ? NaN : v) ∘ log10 ∘ abs

    ref_Res, ref_us = [], Vector[]
    for idx = 1:nrow(df)
        rs  = vec_rs[idx]
        t   = vec_t[idx]
        Res = vec_Res[idx]
        us  = vec_us[idx]
        for (ri,Rei,ui) in zip(rs, Res, us)
            ri != r && continue
            push!(ref_Res, Rei)
            push!(ref_us, ui)
        end
    end
    @assert length(ref_Res) == length(ref_us) == nrow(df)

    Res_intrp, u_intrp = GWanalysis.interpolate_common_grid(ref_us, [ abs.(Re) for Re in ref_Res ], idx_use_as_ref=1)
    avg_diffRes, diffRes = [], []
    for (i,Re) in enumerate(Res_intrp)
        i < length(Res_intrp) || break
        diffRe = @. trafo(Res_intrp[i+1] .- Res_intrp[i])
        push!(diffRes, diffRe)
        avg_diffRe = moving_average(diffRe, 20)
        push!(avg_diffRes, avg_diffRe)
    end

    fig = Figure(figure_padding=(50,50,0,0))
    ga  = fig[1,1] = GridLayout()
    ax  = Axis(ga[1,1]; xlabel=L"u/M_{\textrm{BBS}}")
    ax.ylabel[] = ylabel
    ax.xtickformat[] = xs -> [ @sprintf "%.0f" x for x in xs ]

    for (i, avg_diffRe) in enumerate(avg_diffRes)
        n1, n2 = df.n[i+1], df.n[i]
        c = Makie.wong_colors()[i]
        cc = RGBAf(c.r,c.g,c.b, i%2 == (length(avg_diffRe)-1)%2 ? 1.0 : 0.6)
        if i == 7
            cc = :black
        end
        ls = i%2==0 ? :dash : :solid
        lines!(ax, u_intrp./M, M .* avg_diffRe, label=L"\Delta n = %$n1 - %$n2", color=cc,
              linestyle=ls)
    end
    lines!(ax, u_intrp./M, M .* diffRes[end], linestyle=:dot, color=(:gray,0.5),
           label="raw")
    # not plotting redarded time here, because we want to indicate which part of the
    # GW signal resulted from the merger
    vlines!(ax, merger_t, color=:black, linestyle=:dash)

    Legend(ga[0,1], ax, orientation=:horizontal, tellwidth=false, width=nothing,
           colgap=30, nbanks=nbanks, labelvalign=:top, labelsize=30)

    rowgap!(ga, 0)

    return fig
end

#######################################################################
#                                Plot                                 #
#######################################################################

let

    M = 0.79
    ylimits = (-11,-3)
    yticks = -11:2:-3
    # xlimits = (500,3700)
    xlimits = nothing
    plt_w, plt_h  = 1200, 800
    radius = 720
    varname = "Psi4_20"

    plotname = joinpath(@__DIR__, "psi4_self_convergence_comparison8_cts_helfer_v_0.05_r_$radius$(EXT)")
    runs = find_runs(OUTPUTS["comparison8/cts_helfer_v_0.05"])
    df = load_runs(runs)

    fig = plot_psi4_diffs(df, M, varname, radius,
                          L"\log_{10} |\Delta (M_{\textrm{BBS}}~\text{Re}(r_{\text{ext}} \Psi_{4,20}))|";
                          xlimits, ylimits, yticks, nbanks=2)

    display(fig)

    @info "saving '$plotname'"
    save(plotname, fig, resolution=(plt_w, plt_h), backend=CairoMakie)

end
