using FFTW

### A small experiment to determine the 'normalization' of the rfft transform
#
# Consider a Cosine signal which oscillations with a frequency of 1 Hz
# in is resolved on a time range of 1 s with 10 equally in time spaced samples,
# e.g. dt = 0.1

f = 1 # Hz
tend = 1 # s
N = 10
dt = tend/N
t = collect(range(0,step=dt,length=N))
u = @. cos(2*pi*f*t)

@show t
@show u


# rfft computes the fast-fourier-transform of a real signal
# in particular, it uses the fact that the Fourier transform of a real signal
# has a symmetry, e.g. ̂u(-f) = ̂u*(f) [ * is complex conjugation, ̂  denotes the Fourier transform ]
uh = rfft(u)

@show uh

# because of the symmetry uh has now the length (I think the +1 factor is so
# that there is always a zero-frequency component, i.g. there is always a constant
# factor in the Fourier series)

@assert length(u)÷2+1 == length(uh)

# when we look at the output of uh, we see that the only non-zero component of it
# (considering everything below 1e-14 as zero) is the second one

@show uh[2]

# note that the amplitude of uh[2] is not equal to one; instead it is equal to
# the length of the Fourier transformed vector; this is due to the convention chosen
# by the FFTW.jl library (or rather the AbstractFourierTransform.jl interface, I think)

@show uh[2] / length(uh) ≈ 1

# the corresponding frequency bining of this discrete fourier transform
# can be obtained with the rfftfreq function; note that this function needs the
# length of the input that was used for the rfft function, because the length
# of the frequency vector should agree with that of uh
fs = rfftfreq(length(u),1/dt)

@show fs

# if we now look at the second element of fs (remember the it was the second element
# of uh which was only non-zero) we see that the frequency corresponding to the
# only non-zero DFT component is at frequency 1

@show fs[2]
