using GWanalysis
using DataFrames
using Printf
using CairoMakie
using GLMakie
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")
include("utils.jl")


GLMakie.closeall()

let

# bare scalar field mass
mu = 1e-11
# frequency window for wiener product
fmin, fmax = 5, 1000 # Hz

plt_w, plt_h = 1200, 1300
ylims = (-5.8, 0.7)
yticks = -6:1

plotname = joinpath(@__DIR__, "psi4_wienerprod_comparison_all_vs$(EXT)")

all_groups = []
boosts = [ "0.05", "0.1", "0.15" ]
# boosts = [ "0.15" ]
for b in boosts
    groups = [
              (OUTPUTS["comparison8/cts_helfer_v_$b"], OUTPUTS["comparison8/cts_naive_v_$b"]),
              (OUTPUTS["comparison8/helfer_v_$b"],     OUTPUTS["comparison8/naive_v_$b"]),
              (OUTPUTS["comparison8/cts_helfer_v_$b"], OUTPUTS["comparison8/helfer_v_$b"]),
              (OUTPUTS["comparison8/cts_naive_v_$b"],  OUTPUTS["comparison8/naive_v_$b"]),
    ]
    push!(all_groups, groups)
end
labels = [
    ("CTS+CVE", "CTS+SSP"),
    ("CVE", "SSP"),
    ("CTS+CVE", "CVE"),
    ("CTS+SSP", "SSP"),
]
linestyles = [
    :solid, :dash, :solid, :solid
]
markersize = [
    35, 20, 20, 20
]
markers = [
    :rect, :star4, :diamond, :circle
]


#######################################################################
#                                Plot                                 #
#######################################################################


fig = Figure()
gl = fig[1,1] = GridLayout()
axs = [ Axis(gl[i,1], ylabel=L"\log_{10}(~1 - W~)") for i = 1:length(boosts) ]
axs[end].xlabel[] = L"n"

for (i,b) in enumerate(boosts)
    gg = GridLayout(gl[i,1], tellwidth = false, tellheight = false,
                    halign = :right, valign = :bottom)
    box = Box(gg[1,1], color=:white, strokecolor=:black)
    lbl = Label(gg[1,1], L"v = %$b", padding=(20,20,20,20))
    # bring box and label in front of axis ticks
    translate!(box.blockscene, (0,0,100))
    translate!(lbl.blockscene, (0,0,100))
end

linkaxes!(axs...)
for ax in axs
    ax.yticks[] = yticks
end
ylims!(axs[end], ylims)
for ax in @view axs[1:end-1]
    hidexdecorations!(ax, ticks=false, grid=false, minorgrid=false, minorticks=false)
end

for (ib,(b,groups)) in enumerate(zip(boosts,all_groups))

    groups_mms_1 = []
    groups_mms_2 = []
    groups_mms_3 = []
    groups_mms_4 = []
    groups_mms_5 = []
    groups_Is_1 = []
    groups_Is_2 = []
    groups_Is_3 = []
    groups_Is_4 = []
    groups_Is_5 = []
    n_commons    = []

    for (i,(project1, project2)) in enumerate(groups)
        @info "Analyzing group $(labels[i])"
        runs1 = find_runs(project1)
        runs2 = find_runs(project2)
        df1 = load_runs(runs1)
        df2 = load_runs(runs2)

        GWanalysis.getparameter(df1, Int64, "grid.n.xyz"; newkey="n")
        GWanalysis.getparameter(df2, Int64, "grid.n.xyz"; newkey="n")
        @assert allunique(df1."n")
        @assert allunique(df2."n")
        sort!(df1, :n)
        sort!(df2, :n)

        n_common = intersect(df1.n, df2.n)
        push!(n_commons, n_common)
        @assert !isempty(n_common)
        df1 = filter(df1) do row
            row.n in n_common
        end
        df2 = filter(df2) do row
            row.n in n_common
        end
        display(n_common)


        # load all data
        GWanalysis.get_output_analysis2(df1, "Psi4_20_Nextrp_1")
        GWanalysis.get_output_analysis2(df1, "Psi4_20_Nextrp_2")
        GWanalysis.get_output_analysis2(df1, "Psi4_20_Nextrp_3")
        GWanalysis.get_output_analysis2(df1, "Psi4_20_Nextrp_4")
        GWanalysis.get_output_analysis2(df1, "Psi4_20_Nextrp_5")

        GWanalysis.get_output_analysis2(df2, "Psi4_20_Nextrp_1")
        GWanalysis.get_output_analysis2(df2, "Psi4_20_Nextrp_2")
        GWanalysis.get_output_analysis2(df2, "Psi4_20_Nextrp_3")
        GWanalysis.get_output_analysis2(df2, "Psi4_20_Nextrp_4")
        GWanalysis.get_output_analysis2(df2, "Psi4_20_Nextrp_5")

        # noise whiten 2nd differenced psi4
        f1_1, dd_ft1_1 = compute_dd_psi4(df1."Psi4_20_Nextrp_1:Time", df1."Psi4_20_Nextrp_1:Psi4")
        f1_2, dd_ft1_2 = compute_dd_psi4(df1."Psi4_20_Nextrp_2:Time", df1."Psi4_20_Nextrp_2:Psi4")
        f1_3, dd_ft1_3 = compute_dd_psi4(df1."Psi4_20_Nextrp_3:Time", df1."Psi4_20_Nextrp_3:Psi4")
        f1_4, dd_ft1_4 = compute_dd_psi4(df1."Psi4_20_Nextrp_4:Time", df1."Psi4_20_Nextrp_4:Psi4")
        f1_5, dd_ft1_5 = compute_dd_psi4(df1."Psi4_20_Nextrp_5:Time", df1."Psi4_20_Nextrp_5:Psi4")

        f2_1, dd_ft2_1 = compute_dd_psi4(df2."Psi4_20_Nextrp_1:Time", df2."Psi4_20_Nextrp_1:Psi4")
        f2_2, dd_ft2_2 = compute_dd_psi4(df2."Psi4_20_Nextrp_2:Time", df2."Psi4_20_Nextrp_2:Psi4")
        f2_3, dd_ft2_3 = compute_dd_psi4(df2."Psi4_20_Nextrp_3:Time", df2."Psi4_20_Nextrp_3:Psi4")
        f2_4, dd_ft2_4 = compute_dd_psi4(df2."Psi4_20_Nextrp_4:Time", df2."Psi4_20_Nextrp_4:Psi4")
        f2_5, dd_ft2_5 = compute_dd_psi4(df2."Psi4_20_Nextrp_5:Time", df2."Psi4_20_Nextrp_5:Psi4")

        psd_name = "T1800044-v5.txt"
        f_psd, psd = GWanalysis.load_psd(psd_name)
        # compute 2nd differenced noise curve
        dd_psd = compute_dd_psd(f_psd, psd)
        # limit noise curve to (fmin,fmax) window
        idx_fmin = findfirst(>=(fmin), f_psd) |> something
        idx_fmax = findfirst(>=(fmax), f_psd) |> something
        f_psd    = f_psd[idx_fmin:idx_fmax-1]
        psd      = psd[idx_fmin:idx_fmax-1]
        dd_psd   = dd_psd[idx_fmin:idx_fmax-1]

        mms_1 = compute_psi4_wienerprod_pairs(f1_1, dd_ft1_1, f2_1, dd_ft2_1, f_psd, dd_psd, mu)
        mms_2 = compute_psi4_wienerprod_pairs(f1_2, dd_ft1_2, f2_2, dd_ft2_2, f_psd, dd_psd, mu)
        mms_3 = compute_psi4_wienerprod_pairs(f1_3, dd_ft1_3, f2_3, dd_ft2_3, f_psd, dd_psd, mu)
        mms_4 = compute_psi4_wienerprod_pairs(f1_4, dd_ft1_4, f2_4, dd_ft2_4, f_psd, dd_psd, mu)
        mms_5 = compute_psi4_wienerprod_pairs(f1_5, dd_ft1_5, f2_5, dd_ft2_5, f_psd, dd_psd, mu)

        Is_1 = compute_psi4_inaccuracy_pairs(f1_1, dd_ft1_1, f2_1, dd_ft2_1, f_psd, dd_psd, mu)
        Is_2 = compute_psi4_inaccuracy_pairs(f1_2, dd_ft1_2, f2_2, dd_ft2_2, f_psd, dd_psd, mu)
        Is_3 = compute_psi4_inaccuracy_pairs(f1_3, dd_ft1_3, f2_3, dd_ft2_3, f_psd, dd_psd, mu)
        Is_4 = compute_psi4_inaccuracy_pairs(f1_4, dd_ft1_4, f2_4, dd_ft2_4, f_psd, dd_psd, mu)
        Is_5 = compute_psi4_inaccuracy_pairs(f1_5, dd_ft1_5, f2_5, dd_ft2_5, f_psd, dd_psd, mu)

        push!(groups_mms_1, mms_1)
        push!(groups_mms_2, mms_2)
        push!(groups_mms_3, mms_3)
        push!(groups_mms_4, mms_4)
        push!(groups_mms_5, mms_5)
        push!(groups_Is_1, Is_1)
        push!(groups_Is_2, Is_2)
        push!(groups_Is_3, Is_3)
        push!(groups_Is_4, Is_4)
        push!(groups_Is_5, Is_5)

    end


    ax = axs[ib]
    n_common = intersect(n_commons...)
    xticks = [ @sprintf("%d", n) for n in n_common ]
    ax.xticks[] = (n_common, xticks)

    colors = Makie.wong_colors()

    for i = 1:length(groups)
        c = colors[i]
        lbl1, lbl2 = labels[i]
        mms_1, mms_2, mms_3, mms_4, mms_5 = groups_mms_1[i], groups_mms_2[i], groups_mms_3[i], groups_mms_4[i], groups_mms_5[i]
        ns = n_commons[i]

        scatterlines!(ax, ns, log10.(mms_1), color=c, label="$lbl1 vs. $lbl2", linestyle=linestyles[i], linewidth=2, markersize=markersize[i], marker=markers[i])
        scatterlines!(ax, ns, log10.(mms_2), color=c, linestyle=linestyles[i], linewidth=2, markersize=markersize[i], marker=markers[i])
        scatterlines!(ax, ns, log10.(mms_2), color=c, linestyle=linestyles[i], linewidth=2, markersize=markersize[i], marker=markers[i])
        scatterlines!(ax, ns, log10.(mms_4), color=c, linestyle=linestyles[i], linewidth=2, markersize=markersize[i], marker=markers[i])
        scatterlines!(ax, ns, log10.(mms_2), color=c, linestyle=linestyles[i], linewidth=2, markersize=markersize[i], marker=markers[i])
    end

end

Legend(gl[0,1], axs[1], orientation=:horizontal,
       width=nothing, tellwidth=false, colgap=35, nbanks=2, patchsize=(70,20))
colgap!(gl, 0)
rowgap!(gl, 0)

display(GLMakie.Screen(), fig)
@info "Saving $plotname"
save(plotname, fig, resolution=(plt_w,plt_h), backend=CairoMakie)



end
