using FFTW


# no factor of 2pi needed here, because
# - rfftfreq(length(u),1/dt) already returns a frequency vector (and no angular frequency vector!)
#   cf. mwe_rfft.jl
# - the conversion factors between the dimensionless rescaling and dimensionful variables are given by
#   ω0 = f0 = 1/t0 and
#   f = hat{f} f0 = hat{f} ω0 = hat{ω}/2π ω0
# - from the Helfer paper we extract the value
#   ω0 = (mu / 6.582e-16 eV) Hz
f_to_SI(mu) = mu / 6.582e-16


psi4_match(df, psi1, psi2, psd) = 4 * real(inner_product(df, psi1, psi2, psd))


# compute fourier transform of second differenced NR signal
# following arXiv:2205.15029
function compute_dd_psi4(us, Psi4s)
    @assert length(us) == length(Psi4s)

    dd_ft_Psi4s = []
    fs = []
    for i in eachindex(us, Psi4s)
        u, Psi4 = us[i], Psi4s[i]
        ft = rfft(Psi4)
        dt = u[2]-u[1]
        f = rfftfreq(length(u), 1/dt)
        df = 1/(length(u)*dt)
        dd_ft = [ (1 - cos(2*pi*k*df*dt)) / (2*pi^2*(k*df*dt)^2) * ft[k] for k = 1:length(f) ]
        push!(dd_ft_Psi4s, dd_ft)
        push!(fs, f)
    end

    return fs, dd_ft_Psi4s
end


# compute second differenced detector PSD following arXiv:2205.15029
function compute_dd_psd(f_psd, psd)

    df_min, df_max = extrema(diff(f_psd))
    @assert df_min - df_max <= 1e-3
    df = (df_min + df_max) / 2
    M = length(f_psd)
    dt = 1/df
    T = M * dt

    k1 = Int64(ceil(f_psd[1]/df))

    ks = k1:k1+M-1
    # the expression can underflow when M is really big (~500000)
    factor = @. abs((6 - 8 * cos(2*pi*ks/M) + 2 * cos(4*pi*ks/M)))
    dd_psd = @. factor * psd

    return dd_psd
end


# compute psi4 wiener product for a series of signals
# with data
#   fs     = [ f1,         f2, ... ]
#   dd_fts = [ dd_ft1, dd_ft2, ... ]
function compute_psi4_wienerprod_series(fs, dd_fts, f_psd, psd, mu)

    N = length(fs)
    @assert length(dd_fts) == N

    mms = zeros(Float64, N-1)
    for i = 1:N-1
        f1, f2 = f_to_SI(mu) .* fs[i], f_to_SI(mu) .* fs[i+1]
        ft1, ft2 = dd_fts[i], dd_fts[i+1]
        # _, idxmax1 = findmax(ft1)
        # _, idxmax2 = findmax(ft2)
        # @info (f1[idxmax1],f2[idxmax2])
        data_intrp, f_common =
            GWanalysis.interpolate_common_grid([f1,f1,f2,f2,f_psd],
                                               [real.(ft1),imag.(ft1),real.(ft2),imag.(ft2),psd],
                                               use_max_dx=true)
        rft1, ift1, rft2, ift2, ipsd = data_intrp
        intrp_ft1, intrp_ft2 = rft1 .+ im.*ift1, rft2 .+ im.*ift2
        df = f_common[2]-f_common[1]
        norm_intrp_ft1 = psi4_match(df, intrp_ft1, intrp_ft1, ipsd)
        norm_intrp_ft2 = psi4_match(df, intrp_ft2, intrp_ft2, ipsd)
        intrp_ft1 ./= sqrt(norm_intrp_ft1)
        intrp_ft2 ./= sqrt(norm_intrp_ft2)
        mms[i] = 1.0 - psi4_match(df, intrp_ft1, intrp_ft2, ipsd)
    end

    return mms
end


# compute psi4 wiener product between two series of signals
# with data
#   f1s     = [ f11,         f12, ... ]
#   dd_ft1s = [ dd_ft11, dd_ft12, ... ]
# and
#   f2s     = [ f21,         f22, ... ]
#   dd_ft2s = [ dd_ft21, dd_ft22, ... ]
function compute_psi4_wienerprod_pairs(f1s, dd_ft1s, f2s, dd_ft2s, f_psd, psd, mu)

    N = length(f1s)
    @assert length(dd_ft1s) == length(dd_ft2s) == length(f2s) == N

    mms = zeros(Float64, N)
    for i = 1:N
        f1, f2 = f_to_SI(mu) .* f1s[i], f_to_SI(mu) .* f2s[i]
        ft1, ft2 = dd_ft1s[i], dd_ft2s[i]
        # _, idxmax1 = findmax(real.(ft1))
        # _, idxmax2 = findmax(real.(ft2))
        # @info (f1[idxmax1],f2[idxmax2])
        data_intrp, f_common =
            GWanalysis.interpolate_common_grid([f1,f1,f2,f2,f_psd],
                                               [real.(ft1),imag.(ft1),real.(ft2),imag.(ft2),psd],
                                               use_max_dx=true)
        rft1, ift1, rft2, ift2, ipsd = data_intrp
        intrp_ft1, intrp_ft2 = rft1 .+ im.*ift1, rft2 .+ im.*ift2
        df = f_common[2]-f_common[1]
        norm_intrp_ft1 = psi4_match(df, intrp_ft1, intrp_ft1, ipsd)
        norm_intrp_ft2 = psi4_match(df, intrp_ft2, intrp_ft2, ipsd)
        intrp_ft1 ./= sqrt(norm_intrp_ft1)
        intrp_ft2 ./= sqrt(norm_intrp_ft2)
        test_norm_intrp_ft1 = psi4_match(df, intrp_ft1, intrp_ft1, ipsd)
        test_norm_intrp_ft2 = psi4_match(df, intrp_ft2, intrp_ft2, ipsd)
        mm = 1.0 - psi4_match(df, intrp_ft1, intrp_ft2, ipsd)
        @info (test_norm_intrp_ft1, test_norm_intrp_ft2, mm)
        if !(0 <= mm <= 1)
            @warn "Invalid wiener product for index i = $i"
        end
        mms[i] = mm
        # mms[i] = 1.0 - psi4_match(df, intrp_ft1, intrp_ft2, ipsd)
    end

    return mms
end


# compute psi4 inaccuracy for a series of signals
# with data
#   fs     = [ f1,         f2, ... ]
#   dd_fts = [ dd_ft1, dd_ft2, ... ]
function compute_psi4_inaccuracy_series(fs, dd_fts, f_psd, psd, mu)

    N = length(fs)
    @assert length(dd_fts) == N

    Is = zeros(Float64, N-1)
    for i = 1:N-1
        f1, f2 = f_to_SI(mu) .* fs[i], f_to_SI(mu) .* fs[i+1]
        ft1, ft2 = dd_fts[i], dd_fts[i+1]
        data_intrp, f_common =
            GWanalysis.interpolate_common_grid([f1,f1,f2,f2,f_psd],
                                               [real.(ft1),imag.(ft1),real.(ft2),imag.(ft2),psd],
                                               use_max_dx=true)
        rft1, ift1, rft2, ift2, ipsd = data_intrp
        intrp_ft1, intrp_ft2 = rft1 .+ im.*ift1, rft2 .+ im.*ift2
        df = f_common[2]-f_common[1]
        norm_intrp_ft1 = psi4_match(df, intrp_ft1, intrp_ft1, ipsd)
        norm_intrp_ft2 = psi4_match(df, intrp_ft2, intrp_ft2, ipsd)
        intrp_ft1 ./= sqrt(norm_intrp_ft1)
        intrp_ft2 ./= sqrt(norm_intrp_ft2)
        match = psi4_match(df, intrp_ft1, intrp_ft2, ipsd)
        Is[i] = (sqrt(norm_intrp_ft1/norm_intrp_ft2) - 1)^2 + 2 * sqrt(norm_intrp_ft1/norm_intrp_ft2) * (1 - match)
    end

    return Is
end


# compute psi4 inaccuracy between two series of signals
# with data
#   f1s     = [ f11,         f12, ... ]
#   dd_ft1s = [ dd_ft11, dd_ft12, ... ]
# and
#   f2s     = [ f21,         f22, ... ]
#   dd_ft2s = [ dd_ft21, dd_ft22, ... ]
function compute_psi4_inaccuracy_pairs(f1s, dd_ft1s, f2s, dd_ft2s, f_psd, psd, mu)

    N = length(f1s)
    @assert length(dd_ft1s) == length(dd_ft2s) == length(f2s) == N

    Is = zeros(Float64, N)
    for i = 1:N
        f1, f2 = f_to_SI(mu) .* f1s[i], f_to_SI(mu) .* f2s[i]
        ft1, ft2 = dd_ft1s[i], dd_ft2s[i]
        data_intrp, f_common =
            GWanalysis.interpolate_common_grid([f1,f1,f2,f2,f_psd],
                                               [real.(ft1),imag.(ft1),real.(ft2),imag.(ft2),psd],
                                               use_max_dx=true)
        rft1, ift1, rft2, ift2, ipsd = data_intrp
        intrp_ft1, intrp_ft2 = rft1 .+ im.*ift1, rft2 .+ im.*ift2
        df = f_common[2]-f_common[1]
        norm_intrp_ft1 = psi4_match(df, intrp_ft1, intrp_ft1, ipsd)
        norm_intrp_ft2 = psi4_match(df, intrp_ft2, intrp_ft2, ipsd)
        intrp_ft1 ./= sqrt(norm_intrp_ft1)
        intrp_ft2 ./= sqrt(norm_intrp_ft2)
        match = psi4_match(df, intrp_ft1, intrp_ft2, ipsd)
        Is[i] = (sqrt(norm_intrp_ft1/norm_intrp_ft2) - 1)^2 + 2 * sqrt(norm_intrp_ft1/norm_intrp_ft2) * (1 - match)
    end

    return Is
end
